/*
Dozen Days of Dice episode 10: General
Copyright (c) 2008 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.dice.general
{
	import flash.display.SimpleButton;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import com.blazinggames.dozendays.*;
	import com.blazinggames.dozendays.dice.*;

	public class GeneralGame
	extends Sprite
	{
// ***** CONSTANTS **********************************************************

		public static const KEEP_STRING:String = "Keeping";
		public static const ROLL_STRING:String = "Re-roll";		
		public static const ROLL_TEXT:String = "Roll!";
//		public static const DONE_TEXT:String = "Done!";
		
// ***** VARIABLES **********************************************************

		protected var _manager:IGameMode;
		protected var _setup:PlayerSetup;
		protected var _rollButton:TextButton;
//		protected var _doneButton:TextButton;
		protected var _toTitle:TextButton;
		protected var _restartGame:TextButton;
		
		protected var _keep:Array;
		protected var _dice:Array;
		protected var _roundScore:int;

		protected var _scoresheets:Array;

		protected var _round:int;
		protected var _player:int;
		protected var _rolls:int;

// ***** CONSTRUCTOR ********************************************************

		public function GeneralGame(manager:IGameMode, setup:PlayerSetup)
		{
			var cntr:int
			_manager = manager;
			_setup = setup;
			render();
			
			_rollButton = new TextButton(ROLL_TEXT, 300, 40);
			_rollButton.x = 10;
			_rollButton.y = 160;
			addChild(_rollButton);
			_rollButton.addEventListener(MouseEvent.MOUSE_UP, handleRoll);
/*			
			_doneButton = new TextButton(DONE_TEXT, 300, 40);
			_doneButton.x = 330;
			_doneButton.y = 160;
			addChild(_doneButton);
			_doneButton.addEventListener(MouseEvent.MOUSE_UP, handleDone);
*/			
			_keep = new Array(5);
			_dice = new Array(5);
			for (cntr = 0;cntr < 5; ++cntr) {
				_keep[cntr] = new TextButton(ROLL_STRING, 100, 40);
				_keep[cntr].x = 14 + 128*cntr;
				_keep[cntr].addEventListener(MouseEvent.MOUSE_UP, handleKeep);
				addChild(_keep[cntr]);
				
				_dice[cntr] = new GenericDie();
				_dice[cntr].x = _keep[cntr].x;
				_dice[cntr].y = 40;
				_dice[cntr].width = 100;
				_dice[cntr].height = 100;
				addChild(_dice[cntr]);
			}

			_scoresheets = new Array(7)
			for (cntr = 0;cntr < 7; ++cntr) {
				if (cntr < 1)
					_scoresheets[cntr] = new GeneralScoresheet(this, cntr-1, 0xFFFFFF,0xFFFFFF);
				else
				_scoresheets[cntr] = new GeneralScoresheet(this, cntr-1, PlayerSetup.PLAYER_COLORS[cntr-1],PlayerSetup.PLAYER_BACKGROUNDS[cntr-1]);
				_scoresheets[cntr].x = 115 + 75 * cntr;
				_scoresheets[cntr].y = 200;
				addChild(_scoresheets[cntr]);
			}
			_scoresheets[0].x = 0

			_toTitle = new TextButton("return to title...", 310, 20, GeneralScoresheet.SCORE_FORMAT);
			_toTitle.y = 460;
			addChild(_toTitle);
			_toTitle.addEventListener(MouseEvent.MOUSE_UP, handleToTitle);

			_restartGame = new TextButton("Start a new Round", 310, 20, GeneralScoresheet.SCORE_FORMAT);
			_restartGame.x = 330;
			_restartGame.y = 460;
			addChild(_restartGame);
			_restartGame.addEventListener(MouseEvent.MOUSE_UP, handleRestart);
			_restartGame.visible = false;
			startGame();
		}

// ***** Functions - Event Handling *****************************************

		private function handleRoll(evt:Event):void
		{
			var cntr:int;
			
			for (cntr = 0; cntr < 5; ++cntr) {
				if (_keep[cntr].getLabel() == ROLL_STRING)
					_dice[cntr].quickRoll();
			}
			_scoresheets[_player].showPotentialScores(_dice[0].getValue(),
					_dice[1].getValue(), _dice[2].getValue(), 
					_dice[3].getValue(), _dice[4].getValue(), _rolls);
			var generalScore:int = _scoresheets[_player].calculateScore(0, _dice[0].getValue(),
					_dice[1].getValue(), _dice[2].getValue(), 
					_dice[3].getValue(), _dice[4].getValue(), _rolls);
			if (generalScore > 0) {
				_scoresheets[_player].goGeneral();
				_round = 10;
				_player = 6;
				nextPlayer();
			}
			++_rolls;
			if (_rolls > 3)
				_rollButton.visible = false;
			for (cntr = 0; cntr < 5; ++cntr)
				_keep[cntr].visible = ((_rolls > 1) && (_rolls < 4));
		}

		private function handleKeep(evt:Event):void
		{
			var btn:TextButton = TextButton(evt.target)
			if (btn.getLabel() == ROLL_STRING)
				btn.setLabel(KEEP_STRING);
			else
				btn.setLabel(ROLL_STRING);
		}

		public function handleToTitle(evt:MouseEvent):void
		{
			_manager.changeMode(1);
		}

		public function handleRestart(evt:MouseEvent):void
		{
			startGame();
		}

		
// ***** Functions - Rendering **********************************************

		private function render():void
		{
			graphics.clear();
			graphics.beginFill(0x100010);
            graphics.drawRect(0, 0, 640, 200);
            graphics.endFill();
		}

// ***** Functions - Gameplay management ************************************

		public function startGame():void
		{
			_restartGame.visible = false;
			for (var cntr:int = 0; cntr < 6; ++cntr) {
				_scoresheets[cntr+1].setActive(false);
				_scoresheets[cntr+1].clearScores();
			}
			_round = 1;
			_player = 0;
			nextPlayer();
		}
		
		public function nextPlayer():void
		{
			var cntr:int;
			
			_rollButton.visible = true;
			if (_player > 0)
				_scoresheets[_player].setActive(false);
			_rolls = 1;
			for (cntr = 0; cntr < 5; ++cntr) {
				_keep[cntr].setLabel(ROLL_STRING);
				_keep[cntr].visible = ((_rolls > 1) && (_rolls < 4));
			}
			++_player;
			if (_player > 6) {
				_player = 1;
				++_round;
				if (_round > 10) {
					_rollButton.visible = false;
					_restartGame.visible = true;
					for (cntr = 0; cntr < 5; ++cntr)
						_keep[cntr].visible = false;
			
					var winner:int = 1;
					for (cntr = 2; cntr < 7; ++cntr)
						if (_scoresheets[cntr].getTotal() > _scoresheets[winner].getTotal())
							winner = cntr;
					_scoresheets[winner].setActive(true);
					return;
				}
			}
			if (_setup.getPlayerType(_player-1) == PlayerSetup.PLAYER_COMPUTER){
				for (cntr = 0; cntr < 5; ++cntr)
					_dice[cntr].quickRoll();
				_scoresheets[_player].AIScore(_dice[0].getValue(),
					_dice[1].getValue(), _dice[2].getValue(), 
					_dice[3].getValue(), _dice[4].getValue(), _rolls);
			
			} else if (_setup.getPlayerType(_player-1) == PlayerSetup.PLAYER_NONE)
				nextPlayer()
			
			_scoresheets[_player].setActive(true);
		}
		
	} // class
} // package
