/*
Dozen Days of Dice episode 10: General
Copyright (c) 2008 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.dice.general
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import com.blazinggames.dozendays.*;

	public class GeneralScoresheet
	extends Sprite
	{
// ***** CONSTANTS **********************************************************

		public static const LABELS:Array = new Array(
				"General", "Small General", "Four of a Kind",
				"Full house", "Straight", 
				"Sixes", "Fives", "Fours", "Threes", "Twos", "Aces",
				"TOTAL", " ");
		public static const GENERAL:int = 0;
		public static const SMALL_GENERAL:int = 1;
		public static const FOUR_OF_A_KIND:int = 2;
		public static const FULL_HOUSE:int = 3;
		public static const STRAIGHT:int = 4;
		public static const SIXES:int = 5;
		public static const FIVES:int = 6;
		public static const FOURS:int = 7;
		public static const THREES:int = 8;
		public static const TWOS:int = 9;
		public static const ACES:int = 10;
		public static const TOTAL:int = 11;
		public static const LABEL:int = 12;
		
		public static const SCORE_FORMAT:TextFormat = new TextFormat(null, 18, 0x0FFFFFF, null, null, null, null, null, TextFormatAlign.CENTER);
		public static const LABEL_FORMAT:TextFormat = new TextFormat(null, 18, null, true, null, null, null, null, TextFormatAlign.LEFT);

// ***** VARIABLES **********************************************************

		public var _game:GeneralGame;
		protected var _scores:Array;
		protected var _scoreTextFields:Array;
		protected var _scoreButtons:Array;

		protected var _player:int;
		protected var _backColor:uint;
		protected var _forColor:uint;
		protected var _active:Boolean;

// ***** CONSTRUCTOR ********************************************************

		public function GeneralScoresheet(game:GeneralGame, player:int, forColor:uint, backColor:uint)
		{
			_game = game;
			var cntr:int;
			_scores = new Array(13);
			_scoreTextFields = new Array(13);

			_player = player;
			if (player < 0) {
				for (cntr = 0; cntr < 13; ++cntr) {
					_scoreTextFields[cntr] = new TextField();
					_scoreTextFields[cntr].width = 190;
					_scoreTextFields[cntr].height = 20;
					_scoreTextFields[cntr].y = 20 + 20 * cntr;
					_scoreTextFields[cntr].defaultTextFormat = LABEL_FORMAT;
					_scoreTextFields[cntr].text = LABELS[cntr];
					addChild(_scoreTextFields[cntr]);
				}
			} else {
				for (cntr = 0; cntr < 13; ++cntr) {
					_scoreTextFields[cntr] = new TextField();
					_scoreTextFields[cntr].width = 75;
					_scoreTextFields[cntr].height = 20;
					_scoreTextFields[cntr].y = 20 + 20 * cntr;
					_scoreTextFields[cntr].text = " ";
					_scoreTextFields[cntr].defaultTextFormat = SCORE_FORMAT;
					addChild(_scoreTextFields[cntr]);
				}
				clearScores();
			}
			_scoreTextFields[12].y = 0;
			
			_scoreButtons = new Array(11);
			for (cntr = 0; cntr < 11; ++cntr) {
				_scoreButtons[cntr] = new TextButton("0", 75,20, SCORE_FORMAT);
				_scoreButtons[cntr].visible = false;
				_scoreButtons[cntr].y = 20 + 20 * cntr;
				_scoreButtons[cntr].addEventListener(MouseEvent.MOUSE_UP, handleScoreClick);
				addChild(_scoreButtons[cntr]);
			}
			_backColor = backColor;
			_forColor = forColor;
			_active = false;
			render();
		}

		public function clearScores():void
		{
			for (var cntr:int = 0; cntr < 13; ++cntr) {
				_scores[cntr] = -1;
				_scoreTextFields[cntr].text = " ";
			}
		}

// ***** Functions - Event Handling *****************************************

		private function handleScoreClick(evt:Event):void
		{
			var cat:int = 0;
			var cntr:int;
			var totalScore:int = 0;
			
			for (cntr = 0; cntr < 11; ++cntr) {
				_scoreButtons[cntr].visible = false;
				if (evt.target == _scoreButtons[cntr])
					cat = cntr;
			}
			_scores[cat] = int(_scoreButtons[cat].getLabel());
			_scoreTextFields[cat].text = _scores[cat];
			getTotal();
			_game.nextPlayer();
		}

// ***** Functions - Getting and setting *****

		public function getScore(cat:int):int
		{
			return _scores[cat];
		}
	
		public function setActive(b:Boolean):void
		{
			_active = b;
			render();
		}
		
		public function calculateScore(cat:int, d1:int, d2:int, d3:int, d4:int, d5:int, roll:int):int
		{
			var score:int = 0;
			var cntr:int;
			var rollCount:int = 0;
			
			if (_scores[cat] >=0 )
				return _scores[cat];
			var temp:Array = new Array(d1, d2, d3, d4, d5);
			temp.sort();
			
			switch (cat) {
			case GENERAL:
				if ((roll == 1) && (temp[0] == temp[4]))
					score = 1000;
				break;
			case SMALL_GENERAL:
				if (temp[0] == temp[4])
					score = 60;
				break;
			case FOUR_OF_A_KIND:
				if ((temp[0] == temp[3]) || (temp[1] == temp[4])) {
					if (roll == 1)
						score = 45;
					else
						score = 40;
				}
				break;
			case FULL_HOUSE:
				if (((temp[0]==temp[1]) && (temp[3]==temp[4])) && 
						((temp[2] == temp[1]) || (temp[2] == temp[3]))) {
					if (roll == 1)
						score = 35;
					else
						score = 30;
				}
				break
			case STRAIGHT:
				var straight:Boolean = true;
				for (cntr = 0; cntr < 4; ++cntr)
					if ((temp[cntr]+1) != temp[cntr+1])
						straight = false;
				if (straight) {
					if (roll == 1)
						score = 25;
					else
						score = 20;					
				}
				break;
			case SIXES:
				for (cntr = 0; cntr < 5; ++cntr)
					if (temp[cntr] == 6)
						++rollCount;
				score = 6 * rollCount;
				break;
			case FIVES:
				for (cntr = 0; cntr < 5; ++cntr)
					if (temp[cntr] == 5)
						++rollCount;
				score = 5 * rollCount;
				break;
			case FOURS:
				for (cntr = 0; cntr < 5; ++cntr)
					if (temp[cntr] == 4)
						++rollCount;
				score = 4 * rollCount;
				break;
			case THREES:
				for (cntr = 0; cntr < 5; ++cntr)
					if (temp[cntr] == 3)
						++rollCount;
				score = 3 * rollCount;
				break;
			case TWOS:
				for (cntr = 0; cntr < 5; ++cntr)
					if (temp[cntr] == 2)
						++rollCount;
				score = 2 * rollCount;
				break;
			case ACES:
				for (cntr = 0; cntr < 5; ++cntr)
					if (temp[cntr] == 1)
						++rollCount;
				score = rollCount;
				break;
			}
			
			return score;
		}

		public function goGeneral():void
		{
			_scores[0] = 1000;
		}
		
		public function showPotentialScores(d1:int, d2:int, d3:int, d4:int, d5:int, roll:int):void
		{
			var cntr:int;
			
			for (cntr = 1; cntr < 11; ++cntr) {
				if (_scores[cntr] < 0) {
					_scoreButtons[cntr].setLabel(""+calculateScore(cntr, d1, d2, d3, d4, d5, roll));
					_scoreButtons[cntr].visible = true;
				}
			}
			
		}

		public function AIScore(d1:int, d2:int, d3:int, d4:int, d5:int, roll:int):void
		{
			var cntr:int;
			var choice:int = 1;
			var highScore:int = -1;
			for (cntr = 1; cntr < 11; ++cntr) {
				if (_scores[cntr] < 0) {
					if (highScore < calculateScore(cntr, d1, d2, d3, d4, d5, roll)) {
						choice = cntr;
						highScore = calculateScore(cntr, d1, d2, d3, d4, d5, roll);
					}
				}
			}
			_scores[choice] = highScore;
			_scoreTextFields[choice].text = highScore;
			getTotal();
			_game.nextPlayer();
		}
		
		public function getTotal():int
		{
			var totalScore:int = 0;
			
			for (var cntr:int = 0; cntr < 11; ++cntr) {
				if (_scores[cntr] > 0)
					totalScore += _scores[cntr];
			}
			_scoreTextFields[11].text = totalScore;
			
			return totalScore;
		}

// ***** Functions - Rendering *****

		private function render():void
		{
			var gap:int = _scoreTextFields[0].width;
			graphics.clear();
			if (_active)
				graphics.beginFill(_forColor);
			else
				graphics.beginFill(_backColor);
            graphics.drawRect(0, 0, gap, 260);
            graphics.endFill();
			graphics.lineStyle(1,0x808080);
			for (var cntr:int = 1; cntr < 13; ++cntr) {
				graphics.moveTo(0, 20*cntr);
				graphics.lineTo(gap, 20*cntr);
			}
		}
		
	} // class
} // package
